@@ -48,6 +48,9 @@ int diversity_kind = DIVERSITY_NONE;
 int diversity_factor = 256;     /* in units of 1/256 */
 int keep_unfeasible = 0;
 
+static int smoothing_half_life = 0;
+static int two_to_the_one_over_hl = 0; /* 2^(1/hl) * 0x10000 */
+
 /* We maintain a list of "slots", ordered by prefix.  Every slot
    contains a linked list of the routes to this prefix, with the
    installed route, if any, at the head of the list. */
@@ -259,7 +262,7 @@ flush_all_routes()
         while(i < route_slots) {
         /* Uninstall first, to avoid calling route_lost. */
             if(routes[i]->installed)
-                uninstall_route(routes[0]);
+                uninstall_route(routes[i]);
             flush_route(routes[i]);
         }
         i--;
@@ -317,9 +320,9 @@ flush_interface_routes(struct interface *ifp, int v4only)
 void
 for_all_routes(void (*f)(struct babel_route*, void*), void *closure)
 {
-    int i;
+    int i, n = route_slots;
 
-    for(i = 0; i < route_slots; i++) {
+    for(i = 0; i < n; i++) {
         struct babel_route *r = routes[i];
         while(r) {
             (*f)(r, closure);
@@ -331,9 +334,9 @@ for_all_routes(void (*f)(struct babel_route*, void*), void *closure)
 void
 for_all_installed_routes(void (*f)(struct babel_route*, void*), void *closure)
 {
-    int i;
+    int i, n = route_slots;
 
-    for(i = 0; i < route_slots; i++) {
+    for(i = 0; i < n; i++) {
         if(routes[i]->installed)
             (*f)(routes[i], closure);
     }
@@ -481,15 +484,26 @@ change_route_metric(struct babel_route *route,
         }
     }
 
+    /* Update route->smoothed_metric using the old metric. */
+    route_smoothed_metric(route);
+
     route->refmetric = refmetric;
     route->cost = cost;
     route->add_metric = add;
+
+    if(smoothing_half_life == 0) {
+        route->smoothed_metric = route_metric(route);
+        route->smoothed_metric_time = now.tv_sec;
+    }
+
     local_notify_route(route, LOCAL_CHANGE);
 }
 
 static void
 retract_route(struct babel_route *route)
 {
+    /* We cannot simply remove the route from the kernel, as that might
+       cause a routing loop -- see RFC 6126 Sections 2.8 and 3.5.5. */
     change_route_metric(route, INFINITY, INFINITY, 0);
 }
 
@@ -570,7 +584,83 @@ update_feasible(struct source *src,
             (src->seqno == seqno && refmetric < src->metric));
 }
 
-/* This returns the feasible route with the smallest metric. */
+void
+change_smoothing_half_life(int half_life)
+{
+    if(half_life <= 0) {
+        smoothing_half_life = 0;
+        two_to_the_one_over_hl = 0;
+        return;
+    }
+
+    smoothing_half_life = half_life;
+    switch(smoothing_half_life) {
+    case 1: two_to_the_one_over_hl = 131072; break;
+    case 2: two_to_the_one_over_hl = 92682; break;
+    case 3: two_to_the_one_over_hl = 82570; break;
+    case 4: two_to_the_one_over_hl = 77935; break;
+    default:
+        /* 2^(1/x) is 1 + log(2)/x + O(1/x^2) at infinity. */
+        two_to_the_one_over_hl = 0x10000 + 45426 / half_life;
+    }
+}
+
+/* Update the smoothed metric, return the new value. */
+int
+route_smoothed_metric(struct babel_route *route)
+{
+    int metric = route_metric(route);
+
+    if(smoothing_half_life <= 0 ||                 /* no smoothing */
+       metric >= INFINITY ||                       /* route retracted */
+       route->smoothed_metric_time > now.tv_sec || /* clock stepped */
+       route->smoothed_metric == metric) {         /* already converged */
+        route->smoothed_metric = metric;
+        route->smoothed_metric_time = now.tv_sec;
+    } else {
+        int diff;
+        /* We randomise the computation, to minimise global synchronisation
+           and hence oscillations. */
+        while(route->smoothed_metric_time <= now.tv_sec - smoothing_half_life) {
+            diff = metric - route->smoothed_metric;
+            route->smoothed_metric += roughly(diff) / 2;
+            route->smoothed_metric_time += smoothing_half_life;
+        }
+        while(route->smoothed_metric_time < now.tv_sec) {
+            diff = metric - route->smoothed_metric;
+            route->smoothed_metric +=
+                roughly(diff) * (two_to_the_one_over_hl - 0x10000) / 0x10000;
+            route->smoothed_metric_time++;
+        }
+
+        diff = metric - route->smoothed_metric;
+        if(diff > -4 && diff < 4)
+            route->smoothed_metric = metric;
+    }
+
+    /* change_route_metric relies on this */
+    assert(route->smoothed_metric_time == now.tv_sec);
+    return route->smoothed_metric;
+}
+
+static int
+route_acceptable(struct babel_route *route, int feasible,
+                 struct neighbour *exclude)
+{
+    if(route_expired(route))
+        return 0;
+    if(feasible && !route_feasible(route))
+        return 0;
+    if(exclude && route->neigh == exclude)
+        return 0;
+    return 1;
+}
+
+/* Find the best route according to the weak ordering.  Any
+   linearisation of the strong ordering (see consider_route) will do,
+   we use sm <= sm'.  We could probably use a lexical ordering, but
+   that's probably overkill. */
+
 struct babel_route *
 find_best_route(const unsigned char *prefix, unsigned char plen, int feasible,
                 struct neighbour *exclude)
@@ -582,16 +672,20 @@ find_best_route(const unsigned char *prefix, unsigned char plen, int feasible,
         return NULL;
 
     route = routes[i];
+    while(route && !route_acceptable(route, feasible, exclude))
+        route = route->next;
+
+    if(!route)
+        return NULL;
 
     r = route->next;
     while(r) {
-        if(!route_expired(r) &&
-           (!feasible || route_feasible(r)) &&
-           (!exclude || r->neigh != exclude) &&
-           (route_metric(r) < route_metric(route)))
+        if(route_acceptable(r, feasible, exclude) &&
+           (route_smoothed_metric(r) < route_smoothed_metric(route)))
             route = r;
         r = r->next;
     }
+
     return route;
 }
 
@@ -599,6 +693,7 @@ void
 update_route_metric(struct babel_route *route)
 {
     int oldmetric = route_metric(route);
+    int old_smoothed_metric = route_smoothed_metric(route);
 
     if(route_expired(route)) {
         if(route->refmetric < INFINITY) {
@@ -615,7 +710,8 @@ update_route_metric(struct babel_route *route)
                                       neigh->ifp->ifindex);
         change_route_metric(route, route->refmetric,
                             neighbour_cost(route->neigh), add_metric);
-        if(route_metric(route) != oldmetric)
+        if(route_metric(route) != oldmetric ||
+           route_smoothed_metric(route) != old_smoothed_metric)
             route_changed(route, route->src, oldmetric);
     }
 }
@@ -729,6 +825,12 @@ update_route(const unsigned char *id,
         if((feasible || keep_unfeasible) && refmetric < INFINITY)
             route->time = now.tv_sec;
         route->seqno = seqno;
+
+        memset(&route->channels, 0, sizeof(route->channels));
+        if(channels_len > 0)
+            memcpy(&route->channels, channels,
+                   MIN(channels_len, DIVERSITY_HOPS));
+
         change_route_metric(route,
                             refmetric, neighbour_cost(neigh), add_metric);
         route->hold_time = hold_time;
@@ -768,6 +870,8 @@ update_route(const unsigned char *id,
         memcpy(route->nexthop, nexthop, 16);
         route->time = now.tv_sec;
         route->hold_time = hold_time;
+        route->smoothed_metric = MAX(route_metric(route), INFINITY / 2);
+        route->smoothed_metric_time = now.tv_sec;
         route->installed = 0;
         memset(&route->channels, 0, sizeof(route->channels));
         if(channels_len > 0)
@@ -809,7 +913,10 @@ send_unfeasible_request(struct neighbour *neigh, int force,
     }
 }
 
-/* This takes a feasible route and decides whether to install it. */
+/* This takes a feasible route and decides whether to install it.
+   This uses the strong ordering, which is defined by sm <= sm' AND
+   m <= m'.  This ordering is not total, which is what causes
+   hysteresis. */
 
 void
 consider_route(struct babel_route *route)
@@ -838,7 +945,8 @@ consider_route(struct babel_route *route)
     if(route_metric(installed) >= INFINITY)
         goto install;
 
-    if(route_metric(installed) >= route_metric(route) + 64)
+    if(route_metric(installed) >= route_metric(route) &&
+       route_smoothed_metric(installed) > route_smoothed_metric(route))
         goto install;
 
     return;
@@ -938,18 +1046,17 @@ route_changed(struct babel_route *route,
               struct source *oldsrc, unsigned short oldmetric)
 {
     if(route->installed) {
-        if(route_metric(route) > oldmetric) {
-            struct babel_route *better_route;
-            better_route =
-                find_best_route(route->src->prefix, route->src->plen, 1, NULL);
-            if(better_route &&
-               route_metric(better_route) <= route_metric(route) - 96)
-                consider_route(better_route);
-        }
+        struct babel_route *better_route;
+        /* Do this unconditionally -- microoptimisation is not worth it. */
+        better_route =
+            find_best_route(route->src->prefix, route->src->plen, 1, NULL);
+        if(better_route && route_metric(better_route) < route_metric(route))
+            consider_route(better_route);
+    }
 
-        if(route->installed)
-            /* We didn't change routes after all. */
-            send_triggered_update(route, oldsrc, oldmetric);
+    if(route->installed) {
+        /* We didn't change routes after all. */
+        send_triggered_update(route, oldsrc, oldmetric);
     } else {
         /* Reconsider routes even when their metric didn't decrease,
            they may not have been feasible before. */
@@ -966,12 +1073,16 @@ route_lost(struct source *src, unsigned oldmetric)
     if(new_route) {
         consider_route(new_route);
     } else if(oldmetric < INFINITY) {
-        /* Complain loudly. */
+        /* Avoid creating a blackhole. */
         send_update_resend(NULL, src->prefix, src->plen);
-        send_request_resend(NULL, src->prefix, src->plen,
-                            src->metric >= INFINITY ?
-                            src->seqno : seqno_plus(src->seqno, 1),
-                            src->id);
+        /* If the route was usable enough, try to get an alternate one.
+           If it was not, we could be dealing with oscillations around
+           the value of INFINITY. */
+        if(oldmetric <= INFINITY / 2)
+            send_request_resend(NULL, src->prefix, src->plen,
+                                src->metric >= INFINITY ?
+                                src->seqno : seqno_plus(src->seqno, 1),
+                                src->id);
     }
 }
 
